@@ -1,15 +1,16 @@
 from django.conf import settings
-from django.contrib.sites.models import get_current_site
+from django.contrib.sites.shortcuts import get_current_site
 from django.http import HttpResponse
 from django.utils import translation
-from django.utils.translation import (ugettext_lazy as _, pgettext,
-                                      check_for_language)
+from django.utils.translation import (
+    check_for_language, gettext_lazy as _, pgettext,
+)
 from django.views.decorators.cache import never_cache
 from django.views.decorators.csrf import csrf_exempt
 from django.views.generic import View
 
-from .gateway import validate_voice_locale
 from ...views.utils import class_view_decorator
+from .gateway import validate_voice_locale
 
 
 @class_view_decorator(never_cache)
