@@ -1,32 +1,25 @@
-from binascii import unhexlify
 import logging
+from binascii import unhexlify
 
 from django.conf import settings
-from django.core.validators import RegexValidator
 from django.db import models
-from django.utils.translation import ugettext_lazy as _
-
+from django.utils.encoding import force_str
+from django.utils.translation import gettext_lazy as _
 from django_otp.models import Device
 from django_otp.oath import totp
 from django_otp.util import hex_validator, random_hex
+from phonenumber_field.modelfields import PhoneNumberField
+
+from .gateways import make_call, send_sms
 
 try:
     import yubiotp
 except ImportError:
     yubiotp = None
 
-from .gateways import make_call, send_sms
-
 
 logger = logging.getLogger(__name__)
 
-phone_number_validator = RegexValidator(
-    code='invalid-phone-number',
-    regex='^(\+|00)',
-    message=_('Please enter a valid phone number, including your country code '
-              'starting with + or 00.'),
-)
-
 PHONE_METHODS = (
     ('call', _('Phone Call')),
     ('sms', _('Text Message')),
@@ -61,16 +54,22 @@ def key_validator(*args, **kwargs):
     return hex_validator()(*args, **kwargs)
 
 
+def random_hex_str(length=20):
+    # Could be removed once we depend on django_otp > 0.7.5
+    return force_str(random_hex(length=length))
+
+
 class PhoneDevice(Device):
     """
     Model with phone number and token seed linked to a user.
     """
-    number = models.CharField(max_length=16,
-                              validators=[phone_number_validator],
-                              verbose_name=_('number'))
+    class Meta:
+        app_label = 'two_factor'
+
+    number = PhoneNumberField()
     key = models.CharField(max_length=40,
                            validators=[key_validator],
-                           default=random_hex,
+                           default=random_hex_str,
                            help_text="Hex-encoded secret key")
     method = models.CharField(max_length=4, choices=PHONE_METHODS,
                               verbose_name=_('method'))
@@ -81,13 +80,6 @@ class PhoneDevice(Device):
             self.method,
         )
 
-    def __eq__(self, other):
-        if not isinstance(other, PhoneDevice):
-            return False
-        return self.number == other.number \
-            and self.method == other.method \
-            and self.key == other.key
-
     @property
     def bin_key(self):
         return unhexlify(self.key.encode())
